@@ -20,7 +20,7 @@ struct svm_problem
 };
 
 enum { C_SVC, NU_SVC, ONE_CLASS, EPSILON_SVR, NU_SVR };	/* svm_type */
-enum { LINEAR, POLY, RBF, SIGMOID, R};	/* kernel_type */
+enum { LINEAR, POLY, RBF, SIGMOID, R, LAPLACE, BESSEL, ANOVA};	/* kernel_type */
 
 struct svm_parameter
 {
@@ -29,7 +29,7 @@ struct svm_parameter
 	double degree;	/* for poly */
 	double gamma;	/* for poly/rbf/sigmoid */
 	double coef0;	/* for poly/sigmoid */
-
+        double lim; /* for bessel kernel */
 	/* these are for training only */
 	double cache_size; /* in MB */
 	double eps;	/* stopping criteria */
@@ -42,6 +42,7 @@ struct svm_parameter
 	int shrinking;	/* use the shrinking heuristics */
 	SEXP expr;
 	SEXP rho;
+	int d;
 };
 
 struct svm_model *svm_train(const struct svm_problem *prob,
